unit opml;
{
  opml unit
  PRE-PRE-BETA VERSION!!

  about opml see, http://www.opml.org/
}

interface
uses
    IdDateTimeStamp,LibXmlParser,utf8xmlpars,contnrs,
  Windows, Messages, SysUtils, Classes, Controls,
  Dialogs, StdCtrls,IdBaseComponent, IdComponent, IdTCPConnection,IdTCPClient, IdHTTP;

type
 TOpmlBody = class(TObject)
 end;

 TOpmlItem = class(TObject)
 private
  FOwner: TObject;
  FAttrs: TStrings;
  FItems: TObjectList;
  function GetOpmlType: string;
  function GetText: string;
  function GetAsXML: string;
 public
  property Owner: TObject read FOwner;
  property OpmlType: string read GetOpmlType;
  property Items: TObjectList read FItems;
  property Attrs: TStrings read FAttrs;
  property Text: string read GetText;
  property AsXML: string read GetAsXML;
  function GetOpmlAttr(AOpmlType: string): string;
  constructor Create(AOwner: TObject);
  destructor Destroy; override;
 end;

 TOpmlHead = class(TObject)
 private
  function GetAsXml: string;
 public
  title: string;
  dateCreated: TDateTime;
  dateModified: TDateTime;
  ownerEmail: string;
  ownerName: string;
  property AsXml: string read GetAsXml;
  constructor Create;
  destructor Destroy; override;
 end;

 TOpml = class(TObject)
  FVersion: string;
  FEncoding: string;
  FFileName: string;
  FParser: TUtf8ToWideXMLParser;
  FHead: TOpmlHead;
  FItems: TObjectList;
  FLogStrings: TStrings;
  function GetFileName: string;
  function GetAsXml: string;
  procedure SetFileName(AFileName: string);
  procedure Parse;
  procedure LogWrite(AMsg: string);
 public
  property LogStrings: TStrings read FLogStrings;
  property Encoding: string read FEncoding;
  property Version: string read FVersion;
  property AsXml: string read GetAsXml;
  property Head: TOpmlHead read FHead;
  property Items: TObjectList read FItems;
  property FileName: string read GetFileName write SetFileName;
  procedure SaveToFile(AFileName: string);
  procedure LoadFromFile(AFileName: string);
  procedure Clear;
  function GetItemIndex(AAttr,AValue: string): integer;
  constructor Create;
  destructor Destroy; override;
 end;

implementation

function GetLocalTZBias: integer;
var
	TZ : TTimeZoneInformation;
begin
	case GetTimeZoneInformation (TZ) of
		TIME_ZONE_ID_STANDARD: Result := TZ.Bias + TZ.StandardBias;
		TIME_ZONE_ID_DAYLIGHT: Result := TZ.Bias + TZ.DaylightBias;
	else
		Result := TZ.Bias * -1;
	end;
end;

function FormatDateTimeRFC822(ADateTime: TDateTime): string;
begin
   with TIdDateTimeStamp.Create(nil) do
   try
    SetFromTDateTime(ADateTime);
    SetTimeZone(GetLocalTZBias);
    Result := AsRFC822;
   finally
    Free;
   end;
end;

function RFC822ToDateTime(ARFC822String: string): TDateTime;
begin
   with TIdDateTimeStamp.Create(nil) do
   try
    SetFromRFC822(ARFC822String);
    Result := AsTDateTime;
   finally
    Free;
   end;
end;



//******************************************
//   TOPMLHead Implement
//******************************************
function TOpmlHead.GetAsXml: string;
const
 OPML_HEAD='<head>'#10
           +'<title>%s</title>'#10
           +'<dateCreated>%s</dateCreated>'#10
           +'<dateModified>%s</dateModified>'#10
           +'<ownerEmail>%s</ownerEmail>'#10
           +'<ownerName>%s</ownerName>'#10
           +'</head>';
begin
    Result := Format(OPML_HEAD,[
                title,
                FormatDateTimeRFC822(dateCreated), FormatDateTimeRFC822(dateModified),
                ownerEmail,ownerName]);
end;

constructor TOpmlHead.Create;
begin
    Title := '';
    ownerName := '';
    ownerEMail := '';
    dateCreated := Now;
    dateModified := Now;
end;

destructor TOpmlHead.Destroy;
begin
  inherited;
end;

//******************************************
//   TOPMLItem Implement
//******************************************
function TOpmlItem.GetAsXML: string;
var
  i: integer;
begin
    Result := '<outline';

    for i:= 0 to Attrs.Count-1 do
      Result := Result + Format(' %s="%s"',[Attrs.Names[i], Attrs.Values[Attrs.Names[i]]]);

    //I am Father
    if (FItems.Count>0) then
    begin
     Result := Result + '>'#10;
     for i := 0 to FItems.Count-1 do
      Result := Result + TOpmlItem(FItems[i]).AsXml+#10;
     Result := Result + #10'</outline>'#10;
    end
    else Result := Result + ' />'
end;

function TOpmlItem.GetText: string;
begin
    Result := GetOpmlAttr('text');
end;

function TOpmlItem.GetOpmlType: string;
begin
    Result := GetOpmlAttr('type');
end;

function TOpmlItem.GetOpmlAttr(AOpmlType: string): string;
var
  i: integer;
begin
    Result := '';
    for i := 0 to FAttrs.Count-1 do
     if (FAttrs.Names[i]=AOpmlType) then
     begin
      Result := FAttrs.Values[AOpmlType];
      Break;
     end;
end;

constructor TOpmlItem.Create(AOwner: TObject);
begin
  inherited Create;
    FOwner := AOwner;
    FAttrs := TStringList.Create;
    FItems := TObjectList.Create;
end;

destructor TOpmlItem.Destroy;
begin
    FItems.Free;
    FAttrs.Free;
  inherited;
end;


//******************************************
//   TOPML Implement
//******************************************
function TOpml.GetFileName: string;
begin
     Result := FFileName;
end;

procedure TOpml.SetFileName(AFileName: string);
begin
     FFileName := AFileName;
end;

procedure TOpml.SaveToFile(AFileName: string);
var
   f: TextFile;
begin
    AssignFile(f,AFileName);
    Rewrite(f);
    try
      Write(f,AsXml);
    finally
     CloseFile(f);
    end;
end;

procedure TOpml.Clear;
begin
    FItems.Clear;
end;

procedure TOpml.Parse;
var
   AOpmlItem: TOpmlItem;
   i: integer;
   InHead,InBody: Boolean;
   AParent: TObject;
begin
   InHead := false; InBody := false;
   AParent := self;
   with FParser do
   begin
     LoadFromFile(FFileName);
     StartScan;
     while Scan do
       // Here the parser tells you what it has found
       case CurPartType of
        ptXmlProlog: FEncoding := CurEncoding;
        ptStartTag,
        ptEmptyTag : // Process Parser.CurName and Parser.CurAttr (see below) fields here
         begin
          if (CurPartType=ptStartTag) then LogWrite('[ '+CurName+' ptStartTag ]')
          else LogWrite('[ '+CurName+' ptEmptyTag ]');

          //OPML root
          if (CurName='opml') then
          begin
           FVersion := CurAttr.Value('version');
          end;

          //OPML Head
          if (CurName='head') then InHead := True;

          //OPML Body
          if (CurName='body') then InBody := True;
          if (InBody) then
          begin
            //OPML outline
            if (CurName='outline') then
            begin
              //if parent outline is comment ignore sub-outline
              AOpmlItem := TOpmlItem.Create(AParent);
              for i := 0 to CurAttr.Count-1 do
               AOpmlItem.Attrs.Values[CurAttr.Name(i)] := CurAttr.Value(i);

              //isCommented is true ignore this outline
              if (AParent is TOpml) then TOpml(AParent).Items.Add(AOpmlItem)
              else TOpmlItem(AParent).Items.Add(AOpmlItem);

              LogWrite(AOpmlItem.AsXML);
            end;
          end;
         end;
        ptEndTag:
         begin
          LogWrite('[ '+CurName+' ptEndTag ]');
          if (CurName='head') then InHead := false;
          if (CurName='body') then InBody := false;
         end;
        ptContent,ptCDATA:
         begin
          LogWrite('[ '+CurName+' ptContent ]');
          if (InHead) then
          begin
           if (CurName='title') then Head.Title := CurContent;
           if (CurName='ownerName') then Head.ownerName := CurContent;
           if (CurName='ownerEmail') then Head.ownerEmail := CurContent;
           if (CurName='dateCreated') then Head.dateCreated := RFC822ToDateTime(CurContent);
           if (CurName='dateModified') then Head.dateModified := RFC822ToDateTime(CurContent);
          end;
          if (InBody) then
          begin
          end;
         end;
      end;
   end;
end;

function TOpml.GetItemIndex(AAttr,AValue: string): integer;
var
  i: integer;
begin
    Result := -1;
    for i := 0 to Items.Count-1 do
    with TOpmlItem(Items[i]) do
     if Attrs.Values[AAttr]=AValue then
     begin
       Result := i;
       Break;
     end;
end;

procedure TOpml.LoadFromFile(AFileName: string);
begin
   if (not FileExists(AFileName)) then
      raise Exception.Create(Format('%s is not eixsts.',[AFileName]));
   FFileName := AFileName;
   FItems.Clear;
   Parse;
end;

function TOpml.GetAsXml: string;
var
  i: integer;
begin
    Result := Format('<?xml version="1.0" encoding="%s" ?>',[Encoding])+#10;
    Result := Result + Format('<opml version="%s">',[Version])+#10;
    Result := Result + Head.AsXml+#10;
    Result := Result + '<body>'#10;
    for i := 0 to Items.Count-1 do
      Result := Result + TOpmlItem(Items[i]).AsXml+#10;
    Result := Result + '</body>'#10'</opml>';
end;

constructor TOpml.Create;
begin
    FLogStrings := TStringList.Create;
    FVersion := '1.0';
    FEncoding := 'UTF-8';
    FileName := '';
    FHead := TOpmlHead.Create;
    FItems := TObjectList.Create;
    FParser := TUtf8ToWideXMLParser.Create;
end;

destructor TOpml.Destroy;
begin
     FItems.Free;
     FHead.Free;
     FParser.Free;
     FLogStrings.Free;
   inherited;
end;

procedure TOpml.LogWrite(AMsg: string);
begin
    FLogStrings.Add(AMsg);
end;

end.
