{
How to get Win32 Version resource

쐬: Yoshinori Morimoto
쐬: 97/10/09 14:25
JeS: Delphi
}

// Version resource object

unit verres;

interface

type
 TVersionResource = class( TObject )
 private
  FHandle: LongInt;     // Instance Handle = hInstance
  FVersionInfo: Pointer;  // Pointer of VersionInfo
  FSize: Word;       // Size of VersionInfo

  FLanguage: Word;     // Language code
  FCharSet: Word;      // Character set code
  FLangChar: string[8];   // String formatted Language and Character

  procedure GetLangCharSet;
  function GetStrInfo( subtype: string ): string;

  function GetFileVersion: string;
  function GetProductVersion: string;
  function GetFileFlags: LongInt;
  function GetFileOS: LongInt;
  function GetFileType: LongInt;
  function GetSubType: LongInt;

  function GetFileOSString: string;
  function GetFileFlagString: string;
  function GetFileTypeString: string;
  function GetSubTypeString: string;
  function GetLanguage: string;
  function GetCharSet: string;

  function GetComment: string;
  function GetCompanyName: string;
  function GetFileDescription: string;
  function GetFileVerStr: string;
  function GetInternalName: string;
  function GetLegalCopyright: string;
  function GetLegalTradeMarks: string;
  function GetOriginalFilename: string;
  function GetPrivateBuild: string;
  function GetProductName: string;
  function GetProductVerStr: string;
  function GetSpecialBuild: string;
  function GetLogType:string;
 public
  constructor Create( filename: string );
  destructor Destroy; override;

  property FileVersion: string read GetFileVersion;
  property ProductVersion: string read GetProductVersion;
  property FileFlags: LongInt read GetFileFlags;
  property FileOS: LongInt read GetFileOS;
  property FileType: LongInt read GetFileType;
  property SubType: LongInt read GetSubType;

  property StrFileOS: string read GetFileOSString;
  property StrFileFlag: string read GetFileFlagString;
  property StrFileType: string read GetFileTypeString;
  property StrSubType: string read GetSubTypeString;
  property Language: string read GetLanguage;
  property CharSet: string read GetCharSet;

  property Comments: string read GetComment;
  property CompanyName: string read GetCompanyName;
  property FileDescription: string read GetFileDescription;
  property FileVerStr: string read GetFileVerStr;
  property InternalName: string read GetInternalName;
  property LegalCopyright: string read GetLegalCopyright;
  property LegalTradeMarks: string read GetLegalTradeMarks;
  property OriginalFilename: string read GetOriginalFilename;
  property PrivateBuild: string read GetPrivateBuild;
  property ProductName: string read GetProductName;
  property ProductVerStr: string read GetProductVerStr;
  property SpecialBuild: string read GetSpecialBuild;
  property LogType: string read GetLogType;
 end;

implementation

uses
    SysUtils, Windows;

const
 LanguageMax = 45;
 CharSetMax = 12;

 LanguageID: array[1..LanguageMax] of Word = (
  $0401, $0402, $0403, $0404, $0405,
  $0406, $0407, $0408, $0409, $040a,
  $040b, $040c, $040d, $040e, $040f,
  $0410, $0411, $0412, $0413, $0414,
  $0415, $0416, $0417, $0418, $0419,
  $041a, $041b, $041c, $041d, $041e,
  $041f, $0420, $0421, $0804, $0807,
  $0809, $080a, $080c, $0810, $0813,
  $0814, $0816, $081a, $0c0c, $100c
 );
 LanguageName: array[1..LanguageMax] of string[33] = (
  'ArA',
  'uKA',
  'J^jA',
  '`IȒ',
  '`FR',
  'f}[N',
  'hCc',
  'MV',
  'AJp',
  'WXyC',
  'tBh',
  'tX',
  'wuC',
  'nK[',
  'ACXh',
  'C^A',
  '{',
  '؍',
  'I_',
  'mEF[|u[N[',
  '|[h',
  'uWn|gK',
  '[g}X',
  '[}jA',
  'VA',
  'NA`A|ZrAiej',
  'XoLA',
  'AojA',
  'XEF[f',
  '^C',
  'gR',
  'EhD[',
  'onT',
  'ȑ̎',
  'XCXnhCc',
  'CMXp',
  'LVRnXyC',
  'xM[ntX',
  'XCXnC^A',
  'xM[nI_',
  'mEF[|j[mVN',
  '|gK',
  'ZrA|NA`AiLj',
  'Ji_ntX',
  'XCXntX'
 );

 CharSetID: array[1..CharSetMax] of Word = (
    0,  932,  949,  950, 1200, 1250,
  1251, 1252, 1253, 1254, 1255, 1256
 );

 CharSetName: array[1..CharSetMax] of String[16] = (
  '7bit ASCII',
  '{',
  '؍',
  'p',
  'Unicode',
  'e2ij',
  'L',
  '',
  'MV',
  'gR',
  'wuC',
  'ArA'
 );


constructor TVersionResource.Create( filename: string );
var
 fname: string;
 Handle: DWORD;
 Flag: Boolean;
begin
 inherited Create;
   fname := filename;
   Handle := 0;

   FSize := GetFileVersionInfoSize( PChar(fname), Handle );
   if FSize = 0 then FHandle := 0
   else begin
    FHandle := hInstance;
    FVersionInfo := AllocMem( FSize );
    Flag := GetFileVersionInfo(PCHar(fname), FHandle, FSize, FVersionInfo );
    if Flag <> True then FHandle := 0
    else GetLangCharSet;
   end;
end;

destructor TVersionResource.Destroy;
begin
     FreeMem(FVersionInfo, FSize);
   inherited Destroy;
end;

procedure TVersionResource.GetLangCharSet;
var
 Ptr: Pointer;
 LangChar: LongInt;
 Len: UINT;
begin
 if FHandle <> 0 then begin
  VerQueryValue( FVersionInfo, '\VarFileInfo\Translation', Ptr, Len );
  LangChar := LongInt(Ptr^);
  FLanguage := LoWord( LangChar );
  FCharSet := HiWord( LangChar );

  FLangChar := IntToHex( FLanguage,4 ) + IntToHex( FCharSet,4 );
 end; { of if }
end;

function TVersionResource.GetStrInfo( subtype:string ):string;
var
 Ptr: Pointer;
 Len: UINT;
 msg: string;
 pmsg: PChar;
 flag: Boolean;
begin
     if FHandle <> 0 then begin
      pmsg := StrAlloc( 256 );
      msg := '\StringFileInfo\'+FLangChar+'\'+subtype;
      StrPCopy( pmsg, msg );
      try
        flag := VerQueryValue( FVersionInfo, pmsg, Ptr, Len );
        if ( flag <> False ) and ( Len <> 0 ) then
         result := StrPas( PChar(Ptr) );
      finally
       StrDispose( pmsg );
      end;
     end;
end;

function TVersionResource.GetLanguage:String;
var
 lc: Integer;
begin
 if FHandle <> 0 then begin
  for lc := 1 to LanguageMax do
   if FLanguage = LanguageID[lc] then
    result := LanguageName[lc];
  end
 else
  result := '`̌';
end;

function TVersionResource.GetCharSet:string;
var
 lc: Integer;
begin
 if FHandle <> 0 then begin
  for lc := 1 to CharSetMax do
   if FCharSet = CharSetID[lc] then
    result := CharSetName[lc];
  end
 else
  result := ''
end;

function TVersionResource.GetFileVersion:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'FileVersion' );
end;

function TVersionResource.GetProductVersion:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'ProductVersion' );
end;

function TVersionResource.GetFileFlags:LongInt;
var
 Ptr: Pointer;
 Len: UINT;
begin
 if FHandle <> 0 then begin
  VerQueryValue( FVersionInfo, '\', Ptr, Len );
  with TvsFixedFileInfo(Ptr^)  do
   result := dwFileFlags and dwFileFlagsMask;
 end else
  result := 0;
end;

function TVersionResource.GetFileFlagString:string;
var
 BufStr: string;
 lBuf: LongInt;
begin
 result := '';
 BufStr := '';
 lBuf := FileFlags;
 if VS_FF_Debug and lBuf <> 0 then
  BufStr := BufStr + 'fobO񂠂,';
 if VS_FF_InfoInferred and lBuf <> 0 then
  BufStr := BufStr + 'o[W͓Iɐ,';
 if VS_FF_Patched and lBuf <> 0 then
  BufStr := BufStr + 't@C͏CĂ,';
 if VS_FF_PreRelease and lBuf <> 0 then
  BufStr := BufStr + 'Jro[W,';
 if VS_FF_PrivateBuild and lBuf <> 0 then
  BufStr := BufStr + 'Ɣ,';
 if VS_FF_SpecialBuild and lBuf <> 0 then
  BufStr := BufStr + 'ʃo[W,';

 if BufStr <> '' then
  result := Copy( BufStr, 1, Length( BufStr )-1 );
end;

function TVersionResource.GetFileOS:LongInt;
var
 Ptr: Pointer;
 Len: UINT;
begin
 if FHandle <> 0 then begin
  VerQueryValue( FVersionInfo, '\', Ptr, Len );
  with TvsFixedFileInfo(Ptr^)  do
   result := dwFileOS;
 end else
  result := 0;
end;

function TVersionResource.GetFileOSString:string;
var
 BufStr: string;
 lBuf:  LongInt;
begin
 BufStr := '';
 lBuf := FileOS;
 if lBuf = VOS_Unknown then
  BufStr := '`';
 if lBuf = VOS_Dos then
  BufStr := 'MS-DOS';
 if lBuf = VOS_OS216 then
  BufStr := '16bit OS/2';
 if lBuf = VOS_OS232 then
  BufStr := '32bit OS/2';
 if lBuf = VOS_NT then
  BufStr := 'Windows NT';
 if lBuf = VOS__Windows16 then
  BufStr := '16bit Windows';
 if lBuf = VOS__PM16 then
  BufStr := '16bit PM';
 if lBuf = VOS__PM32 then
  BufStr := '32bit PM';
 if lBuf = VOS__Windows32 then
  BufStr := '32bit Windows';
 if lBuf = VOS_DOS_Windows16 then
  BufStr := '16bit Windows on DOS';
 if lBuf = VOS_DOS_Windows32 then
  BufStr := '32bit Windows on DOS';
 if lBuf = VOS_OS216_PM16 then
  BufStr := '16bit OS/2 PM';
 if lBuf = VOS_OS232_PM32 then
  BufStr := '32bit OS/2 PM';
 if lBuf = VOS_NT_Windows32 then
  BufStr := '32bit Windows on NT';

 result := BufStr;
end;

function TVersionResource.GetFileType:LongInt;
var
 Ptr: Pointer;
 Len: UINT;
begin
 if FHandle <> 0 then begin
  VerQueryValue( FVersionInfo, '\', Ptr, Len );
  with TvsFixedFileInfo(Ptr^)  do
   result := dwFileType;
 end else
  result := 0;
end;

function TVersionResource.GetFileTypeString:string;
var
 lBuf: LongInt;
begin
 result := '';
 lBuf := FileType;
 if lBuf = vft_UnKnown then
  result := '`';
 if lBuf = vft_APP then
  result := 'AvP[V';
 if lBuf = vft_DLL then
  result := '_Ci~bNNCu';
 if lBuf = vft_Drv then
  result := 'foCXhCo';
 if lBuf = vft_Font then
  result := 'tHgt@C';
 if lBuf = vft_Vxd then
  result := 'zfoCXhCo';
 if lBuf = vft_Static_Lib then
  result := 'X^eBbNNCu';
end;

function TVersionResource.GetSubType:LongInt;
var
 Ptr: Pointer;
 Len: UINT;
begin
 if FHandle <> 0 then begin
  VerQueryValue( FVersionInfo, '\', Ptr, Len );
  with TvsFixedFileInfo(Ptr^)  do
   result := dwFileSubType;
 end else
  result := 0;
end;

function TVersionResource.GetSubTypeString:string;
var
 tmp: LongInt;
begin
 result := '';
 if FHandle <> 0 then begin
  if FileType = vft_Drv then begin
   Tmp := SubType;
   if Tmp = vft2_UnKnown then
    result := 'sȃfoCXhCo';
   if Tmp = vft2_Drv_Comm then
    result := 'COM |[ghCo';
   if Tmp = vft2_Drv_Printer then
    result := 'v^hCo';
   if Tmp = vft2_Drv_Keyboard then
    result := 'L[{[hhCo';
   if Tmp = vft2_Drv_Language then
    result := 'hCo';
   if Tmp = vft2_Drv_Display then
    result := 'fBXvChCo';
   if Tmp = vft2_Drv_Mouse then
    result := '}EXhCo';
   if Tmp = vft2_Drv_Network then
    result := 'lbg[NhCo';
   if Tmp = vft2_Drv_System then
    result := 'VXehCo';
   if Tmp = vft2_Drv_Installable then
    result := 'CXg[\ȃfoCXhCo';
   if Tmp = vft2_Drv_Sound then
    result := 'TEhhCo';
  end;
  if FileType = vft_Font then begin
   Tmp := SubType;
   if Tmp = vft2_UnKnown then
    result := 'FsȃtHg';
   if Tmp = vft2_Font_Raster then
    result := 'X^tHg';
   if Tmp = vft2_Font_Vector then
    result := 'xNgtHg';
   if Tmp = vft2_Font_TrueType then
    result := 'TrueType tHg';
  end;
 end; { of FHandle if }
end;

function TVersionResource.GetComment:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'Comments' );
end;

function TVersionResource.GetCompanyName:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'CompanyName' );
end;

function TVersionResource.GetFileDescription:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'FileDescription' );
end;

function TVersionResource.GetFileVerStr:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'FileVersion' );
end;

function TVersionResource.GetInternalName:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'InternalName' );
end;

function TVersionResource.GetLegalCopyright:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'LegalCopyright' );
end;

function TVersionResource.GetLegalTradeMarks:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'LegalTrademarks' );
end;

function TVersionResource.GetOriginalFilename:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'OriginalFilename' );
end;

function TVersionResource.GetPrivateBuild:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'PrivateBuild' );
end;

function TVersionResource.GetProductName:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'ProductName' );
end;

function TVersionResource.GetProductVerStr:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'ProductVersion' );
end;

function TVersionResource.GetSpecialBuild:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'SpecialBuild' );
end;

function TVersionResource.GetLogType:string;
begin
 result := '';
 if FHandle <> 0 then
  result := GetStrInfo( 'LogType' );
end;

end.
